import com.kelvinluck.util.LogWrapper;
import com.kelvinluck.flickr.Person;
import com.kelvinluck.flickr.Photo;

/**
* Class: com.kelvinluck.flickr.Photoset
* Describes a Photoset on flickr.com
* 
* Author:
* Kelvin Luck
*/
class com.kelvinluck.flickr.Photoset 
{
	 
	/**
	* Variable: id
	* The id of this <Photoset>
	*/
	var id:Number;
	
	/**
	* Variable: owner
	* The <Person> who owns this <Photoset>
	**/
	var owner:Person;
	/**
	* Variable: url
	* The URL to this <Photoset> on flickr.com
	**/
	var url:String;
	/**
	* Variable: primaryPhoto
	* The primary <Photo> for this <Photoset>
	**/
	var primaryPhoto:Photo;
	/**
	* Variable: numPhotos
	* The total number of <Photo>s in this <Photoset>
	**/
	var numPhotos:Number;
	/**
	* Variable: photos
	* An array of <Photo>s - all of the photos in this <Photoset>
	* TODO:
	* Should this be done through a getter and if it is empty then photosetsGetPhotos
	* is automatically called?
	**/
	private var photos:Array;
	/**
	* Variable: title
	* The title of this <Photoset>
	**/
	var title:String;
	/**
	* Variable: description
	* The description of this <Photoset>
	**/
	var description:String;
	
	/**
	* Variable: _photosets
	* A private static Object containing Photoset objects. Used by <getPhotoset> to insure that only 
	* one Photoset is created for each photoset ID returned from flickr.com
	**/
	private static var _photosets:Object = {};
	
	/**
	* Function: Photoset
	* Constructor - do not call directly, is accessed via <Photoset.getPhotoset>
	**/
	private function Photoset(id:Number)
	{
		this.id = id;
		LogWrapper.getLog().debug(this+" instance created");
		clearPhotos();
	}
	
	/**
	* Function: addPhoto
	* Adds a photo to the end of this Photoset's <photos> array and updates <numPhotos>
	* Called internally on a successful response from flickr.photosets.addPhoto.
	* 
	* Parameters:
	* photo				-	The photo to add.
	**/
	function addPhoto(photo:Photo) {
		if (photo == undefined) { // to deal with when we add photos based on <Flickr.photosetsGetContext> and they are at the beginning or end of a photoset.
			return;
		}
		for (var i:Number=0; i<photos.length; i++) {
			if (photos[i].id == photo.id) {
				return;
			}
		}
		photos.push(photo);
		if (numPhotos) {
			numPhotos++;
		} else {
			numPhotos = photos.length;
		}
		photo.addPhotoset(this);
	}
	/**
	* Function: removePhoto
	* Removes a <Photo> from this <Photoset> - called as a result of a call to <Flickr.photosetsRemovePhoto>
	* 
	* Parameters:
	* photo				-	The <Photo> to remove from this <Photoset>
	* 
	* See Also:
	* <Flickr.photosetsRemovePhoto>
	**/
	function removePhoto(photo:Photo)
	{
		for (var i:Number=0; i<photos.length; i++) {
			if (photos[i] == photo) {
				photos.splice(i, 1);
				break;
			}
		}
		photo.removePhotoset(this);
	}
	
	/**
	* Function: editPhotos
	* Replaces this photoset's <photos> array with one made up from the passed photoIds.
	* Called when there is a response from flickr.com to flickr.photosets.editPhotos
	* 
	* Parameters:
	* primaryPhoto			-	The primary <Photo> for this set.
	* photoIds				-	An array of the ids of <Photo>s in this set.
	* 
	* See Also:
	* <Flickr.photosetsEditPhotos>
	**/
	function editPhotos(primaryPhoto:Photo, photoIds:Array)
	{
		this.primaryPhoto = primaryPhoto;
		clearPhotos();
		for (var i:Number=0; i<photoIds.length; i++) {
			var p:Photo = Photo.getPhoto(photoIds[i]);
			if (owner) {
				p.owner = owner;
				owner.addPhoto(p);
			}
			photos.push(p);
			p.addPhotoset(this);
		}
		numPhotos = photos.length;
	}
	/**
	* Function: clearPhotos
	* 
	* Clears this photoset's <photos> array. Used when the photo list is loaded completely e.g. from
	* <Flickr.photosetsGetPhotos>
	**/
	function clearPhotos()
	{
		photos = [];
		numPhotos = 0;
	}
	/**
	* Function: getPhotos
	* 
	* Get's this <Photoset>s <photos>
	* 
	* Returns:
	* The <photos> for this <Photoset>
	**/
	function getPhotos():Array
	{
		return photos;
	}
	
	/**
	* Function: getPhotoset
	* Get's a Photoset object for the given photoset ID.
	* 
	* Consults <_photosets> to make sure that only one Photoset instance is created for each
	* photoset ID from flickr.com
	* 
	* Parameters:
	* id		-	The id of the Photoset you want to get
	**/
	public static function getPhotoset(id:Number):Photoset
	{
		if (_photosets[id] == undefined) {
			_photosets[id] = new Photoset(id);
		}
		return _photosets[id];
	}
	/**
	* Function: remove
	* Removes a given photoset - called when a photoset is deleted through <Flickr.photosetsDelete>.
	* 
	* Parameters:
	* photoset			-	The photoset you want to remove
	**/
	static function remove(photoset:Photoset):Void
	{
		if (_photosets[photoset.id]) {
			delete _photosets[photoset.id];
		}
		delete photoset;
	} 
	/**
	* Function: toString
	**/
	public function toString():String
	{
		return "[com.kelvinluck.flickr.Photoset] "+ id;
	}
}